@@ -153,15 +153,23 @@ func (token Token) DecodedText(contents string) string {
 }
 
 type lexer struct {
-	log       logger.Log
-	source    logger.Source
-	tracker   logger.LineColumnTracker
-	current   int
-	codePoint rune
-	Token     Token
+	log                     logger.Log
+	source                  logger.Source
+	tracker                 logger.LineColumnTracker
+	current                 int
+	codePoint               rune
+	Token                   Token
+	approximateNewlineCount int
+	sourceMappingURL        logger.Span
 }
 
-func Tokenize(log logger.Log, source logger.Source) (tokens []Token) {
+type TokenizeResult struct {
+	Tokens               []Token
+	ApproximateLineCount int32
+	SourceMapComment     logger.Span
+}
+
+func Tokenize(log logger.Log, source logger.Source) TokenizeResult {
 	lexer := lexer{
 		log:     log,
 		source:  source,
@@ -180,11 +188,16 @@ func Tokenize(log logger.Log, source logger.Source) (tokens []Token) {
 	}
 
 	lexer.next()
+	var tokens []Token
 	for lexer.Token.Kind != TEndOfFile {
 		tokens = append(tokens, lexer.Token)
 		lexer.next()
 	}
-	return
+	return TokenizeResult{
+		Tokens:               tokens,
+		ApproximateLineCount: int32(lexer.approximateNewlineCount) + 1,
+		SourceMapComment:     lexer.sourceMappingURL,
+	}
 }
 
 func (lexer *lexer) step() {
@@ -195,6 +208,16 @@ func (lexer *lexer) step() {
 		codePoint = eof
 	}
 
+	// Track the approximate number of newlines in the file so we can preallocate
+	// the line offset table in the printer for source maps. The line offset table
+	// is the #1 highest allocation in the heap profile, so this is worth doing.
+	// This count is approximate because it handles "\n" and "\r\n" (the common
+	// cases) but not "\r" or "\u2028" or "\u2029". Getting this wrong is harmless
+	// because it's only a preallocation. The array will just grow if it's too small.
+	if codePoint == '\n' {
+		lexer.approximateNewlineCount++
+	}
+
 	lexer.codePoint = codePoint
 	lexer.Token.Range.Len = int32(lexer.current) - lexer.Token.Range.Loc.Start
 	lexer.current += width
@@ -405,12 +428,30 @@ func (lexer *lexer) next() {
 }
 
 func (lexer *lexer) consumeToEndOfMultiLineComment(startRange logger.Range) {
+	// Keep track of the contents of the "sourceMappingURL=" comment
+	startOfSourceMappingURL := 0
+	switch lexer.codePoint {
+	case '#', '@':
+		if strings.HasPrefix(lexer.source.Contents[lexer.current:], " sourceMappingURL=") {
+			startOfSourceMappingURL = lexer.current + len(" sourceMappingURL=")
+		}
+	}
+
 	for {
 		switch lexer.codePoint {
 		case '*':
+			endOfSourceMappingURL := lexer.current - 1
 			lexer.step()
 			if lexer.codePoint == '/' {
 				lexer.step()
+				if startOfSourceMappingURL != 0 {
+					r := logger.Range{Loc: logger.Loc{Start: int32(startOfSourceMappingURL)}}
+					text := lexer.source.Contents[startOfSourceMappingURL:endOfSourceMappingURL]
+					for int(r.Len) < len(text) && !isWhitespace(rune(text[r.Len])) {
+						r.Len++
+					}
+					lexer.sourceMappingURL = logger.Span{Text: text[:r.Len], Range: r}
+				}
 				return
 			}
 
