@@ -41,6 +41,17 @@ func (p *parser) skipTypeScriptBinding() {
 			foundIdentifier := false
 
 			switch p.lexer.Token {
+			case js_lexer.TDotDotDot:
+				p.lexer.Next()
+
+				if p.lexer.Token != js_lexer.TIdentifier {
+					p.lexer.Unexpected()
+				}
+
+				// "{...x}"
+				foundIdentifier = true
+				p.lexer.Next()
+
 			case js_lexer.TIdentifier:
 				// "{x}"
 				// "{x: y}"
@@ -388,7 +399,11 @@ func (p *parser) skipTypeScriptTypeWithOpts(level js_ast.L, opts skipTypeOpts) {
 				p.lexer.Expect(js_lexer.TIdentifier)
 			}
 			p.lexer.Next()
-			p.skipTypeScriptTypeArguments(false /* isInsideJSXElement */)
+
+			// "{ <A extends B>(): c.d \n <E extends F>(): g.h }" must not become a single type
+			if !p.lexer.HasNewlineBefore {
+				p.skipTypeScriptTypeArguments(false /* isInsideJSXElement */)
+			}
 
 		case js_lexer.TOpenBracket:
 			// "{ ['x']: string \n ['y']: string }" must not become a single type
